#pragma once

// Macros suck but I need constant expressions for switch statements. :-(

#define MAKE64BITVERSIONNUMBER(a,b,c,d) ((((DWORD64)(a))<<48) + (((DWORD64)(b))<<32) + (((DWORD64)(c))<<16) + (((DWORD64)(d))<<00))
#define MAKE_DWORD_NAME(a,b,c,d) (((DWORD)(unsigned char)(d)<<24)+((DWORD)(unsigned char)(c)<<16)+((DWORD)(unsigned char)(b)<<8)+((DWORD)(unsigned char)(a)))


namespace LeoHelpers
{
	bool IsWindowsVersionGreaterOrEqual(DWORD dwMaj, DWORD dwMin, WORD wSPMaj, WORD wSPMin);

	inline bool IsWindowsXPOrAbove()
	{
		// 5.0.0.0 would mean Windows Server 2003, Windows XP, or Windows 2000
		// 5.1.0.0 means Windows XP
		return IsWindowsVersionGreaterOrEqual(5,1,0,0);
	}

	inline bool IsWindowsVistaOrAbove()
	{
		// 6.0.0.0 means Windows Vista or Windows Server 2008
		return IsWindowsVersionGreaterOrEqual(6,0,0,0);
	}

	inline bool IsWindows7OrAbove()
	{
		// 6.1.0.0 means Windows 7 or Windows Server 2008 R2
		return IsWindowsVersionGreaterOrEqual(6,1,0,0);
	}

	inline DWORD SwapDWORD(DWORD dw)
	{
		return ((dw<<24)&0xFF000000)
			 + ((dw<<8 )&0x00FF0000)
			 + ((dw>>8 )&0x0000FF00)
			 + ((dw>>24)&0x000000FF);
	}

	inline WORD SwapWORD(WORD w)
	{
		return ((w<<8 )&0xFF00)
			 + ((w>>8 )&0x00FF);
	}

	inline int MulDivRoundDown(int nNumber, int nNumerator, int nDenominator)
	{
		assert(nDenominator != 0);

		if (nDenominator == 0)
		{
			return -1; // Do what MulDiv does.
		}

		__int64 x = nNumber;
		x *= nNumerator;
		x /= nDenominator;

		assert(x <= INT_MAX && x >= INT_MIN);

		if (x > INT_MAX || x < INT_MIN)
		{
			return -1;
		}

		return static_cast< int >(x);
	}

	inline bool SetClipboard(HWND hwndClipboardOwner, const wchar_t *sz)
	{
		bool bResult = false;

		size_t cchBufLen = wcslen(sz) + 1;

		size_t cbBufSize = cchBufLen * sizeof(wchar_t);

		HGLOBAL hGlobalClipMem = ::GlobalAlloc(GMEM_MOVEABLE, cbBufSize);

		if (hGlobalClipMem != NULL)
		{
			wchar_t *pGlobalClipBuffer = reinterpret_cast<wchar_t *>( ::GlobalLock(hGlobalClipMem) );

			if (NULL != pGlobalClipBuffer)
			{
				LeoHelpers::StringCopy(pGlobalClipBuffer, sz, cchBufLen);

				::GlobalUnlock(hGlobalClipMem);

				if (::OpenClipboard(hwndClipboardOwner))
				{
					if (::EmptyClipboard())
					{
						if (NULL != ::SetClipboardData(CF_UNICODETEXT, hGlobalClipMem))
						{
							hGlobalClipMem = NULL; // Clipboard owns it now.
							bResult = true;
						}
					}

					if (!::CloseClipboard())
					{
						bResult = false;
					}
				}
			}

			if (hGlobalClipMem != NULL)
			{
				::GlobalFree(hGlobalClipMem);
				hGlobalClipMem = NULL;
			}
		}

		return bResult;
	}

	inline bool GetClipboard(HWND hwndClipboardOwner, std::wstring *pstrOut, bool bPrefixUTF16BOM, bool *pbOpenFailed)
	{
		bool bResult = false;

		if (pbOpenFailed)
		{
			*pbOpenFailed = false;
		}

		if (!::OpenClipboard(hwndClipboardOwner))
		{
			if (pbOpenFailed)
			{
				*pbOpenFailed = true;
			}
		}
		else
		{
			if (::IsClipboardFormatAvailable(CF_UNICODETEXT))
			{
				HANDLE hGlobalClipMem = ::GetClipboardData(CF_UNICODETEXT);

				if (hGlobalClipMem != NULL)
				{
					const wchar_t *szClipboard = reinterpret_cast<const wchar_t *>( ::GlobalLock(hGlobalClipMem) );

					if (szClipboard)
					{
						const wchar_t szIntelBom[2] = { 0xFEFF, 0 };

						try // We really don't want an exception from std::wstring to stop us closing the clipboard.
						{
							if (bPrefixUTF16BOM && szClipboard[0] != szIntelBom[0])
							{
								*pstrOut += szIntelBom;
							}

							*pstrOut += szClipboard;
							bResult = true;
						}
						catch(...)
						{
						}

						::GlobalUnlock(hGlobalClipMem);
					}
				}
			}

			::CloseClipboard();
		}

		if (!bResult)
		{
			pstrOut->clear();
		}

		return bResult;
	}

	// This requires both a section and a key name. It won't get you a list of names like the real GetPrivateProfileString.
	// Blank values will never be returned and are considered errors.
	inline bool GetPrivateProfileString(std::wstring *pstrResult, const wchar_t *szFileName, const wchar_t *szSectionName, const wchar_t *szKeyName)
	{
		if (szFileName    == NULL || szFileName[0]    == L'\0'
		||	szSectionName == NULL || szSectionName[0] == L'\0'
		||	szKeyName     == NULL || szKeyName[0]     == L'\0')
		{
			return false;
		}

		bool bResult = false;

		DWORD dwBufferSize = 512;
		wchar_t *szBuffer = new(std::nothrow) wchar_t[dwBufferSize];

		while(true)
		{
			DWORD dwRes = ::GetPrivateProfileString(szSectionName, szKeyName, L"", szBuffer, dwBufferSize, szFileName);

			if (dwRes == (dwBufferSize - 1))
			{
				delete [] szBuffer;
				dwBufferSize *= 2;
				szBuffer = new(std::nothrow) wchar_t[dwBufferSize];
			}
			else
			{
				if (dwRes != 0)
				{
					*pstrResult = szBuffer;
					bResult = true;
				}

				break;
			}
		}

		delete [] szBuffer;

		return bResult;
	}

	// This loads strings using the Win32 ::LoadString.
	// There's a separate OpusStringLoader that is similar but loads strings from Directory Opus resources.
	class StringLoader
	{
	private:
		HINSTANCE m_hInstance;
		// Map is to pointers rather than strings so that it doesn't matter if the map nodes move around in memory.
		std::map< UINT, std::wstring * > m_mapStrings;
	public:
		StringLoader(HINSTANCE hInstance) : m_hInstance(hInstance)
		{
		}

		~StringLoader()
		{
			for (std::map< UINT, std::wstring * >::iterator miter = m_mapStrings.begin(); miter != m_mapStrings.end(); ++miter)
			{
				std::wstring *pwstr = miter->second;
				delete pwstr;
			}
		}

		const wchar_t *Get(UINT id)
		{
			std::wstring *&pwstr = m_mapStrings[id];

			if (pwstr == NULL)
			{
				wchar_t szBuffer[1024]; // Assume 1KB is enough for all our strings.

				int res = ::LoadString(m_hInstance, id, szBuffer, _countof(szBuffer)-1);

				if (res <= 0 || res >= _countof(szBuffer))
				{
					_CrtDbgBreak();
					return L"<missing string>";
				}

				szBuffer[res] = L'\0'; // Ensure the buffer is null-terminated. MSDN is unclear about what happens if the string is the same as the buffer length (i.e. no room for the null terminator) which is why we subtracted 1 from the size above.

				pwstr = new std::wstring(szBuffer);
			}

			return pwstr->c_str();
		}
	};
};
