
// Reader_x64_fixerDlg.cpp : implementation file
//

#include "stdafx.h"
#include "LeoHelpers_String.h"
#include "LeoHelpers_Misc.h"
#include "LeoHelpers_Reg_Read.h"
#include "LeoHelpers_Reg_Write.h"
#include "Reader_x64_fixer.h"
#include "Reader_x64_fixerDlg.h"

// We're a 32-bit app so that we can give an error message, and still show our description to,
// people who mistakenly run us under 32-bit Windows. Being a 32-bit app is fine as we can
// still access the 64-bit registry via the KEY_WOW64_64KEY Reg-SAM.
const wchar_t * const CReader_x64_fixerDlg::s_szAdobePreviewHandlerClsid = L"{DC6EFB56-9CFA-464D-8880-44885D7DC193}";
const wchar_t * const CReader_x64_fixerDlg::s_szAdobePreviewHandlerClsidWithPath = L"SOFTWARE\\Classes\\CLSID\\{DC6EFB56-9CFA-464D-8880-44885D7DC193}";
const wchar_t * const CReader_x64_fixerDlg::s_szAppIDWrong = L"{6d2b5079-2f0b-48dd-ab7f-97cec514d30b}";
const wchar_t * const CReader_x64_fixerDlg::s_szAppIDRight = L"{534A1E02-D58F-44f0-B58B-36CBED287C7C}";

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CReader_x64_fixerDlg dialog

// From http://blogs.msdn.com/oldnewthing/archive/2005/02/01/364563.aspx
bool CReader_x64_fixerDlg::Is64BitWindows()
{
#if defined(_WIN64)
	return true;  // 64-bit programs run only on Win64
#elif defined(_WIN32)
	// 32-bit programs run on both 32-bit and 64-bit Windows so must sniff
	BOOL f64 = FALSE;
	return (::IsWow64Process(::GetCurrentProcess(), &f64) && f64);
#else
	return false; // Win64 does not support Win16
#endif
}

CReader_x64_fixerDlg::CReader_x64_fixerDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CReader_x64_fixerDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CReader_x64_fixerDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CReader_x64_fixerDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BUTTON_WEBSITE, &CReader_x64_fixerDlg::OnBnClickedButtonWebsite)
	ON_BN_CLICKED(IDOK, &CReader_x64_fixerDlg::OnBnClickedOk)
	ON_BN_CLICKED(IDC_BUTTON_UNDO, &CReader_x64_fixerDlg::OnBnClickedButtonUndo)
	ON_BN_CLICKED(IDC_CHECK_OFFICE2010BETA, &CReader_x64_fixerDlg::OnBnClickedCheckOffice2010beta)
END_MESSAGE_MAP()

// CReader_x64_fixerDlg message handlers

BOOL CReader_x64_fixerDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	GetDlgItem(IDC_STATIC_ABOUT)->SetWindowText(L"This is a fix for the Adobe Reader preview handler on 64-bit systems.\n\nThe Adobe Reader preview handler allows you to view PDF files within programs such as Outlook 2007/2010, Windows Explorer (Vista && above) and Directory Opus (fix built-in).\n\nThe fix is a simple registry change. See the website linked below for full details.\n\nIMPORTANT: This fix should no longer be needed if you install && update to Adobe Reader 9.3.2 or above. After half a year, Adobe finally fixed their installer. Note that you have to update Adobe Reader after installing it to actually get the latest version.\n\nTHUMBNAILS: A separate fix for PDF thumbnails is still needed. See the website.");

	UpdateAppIDStatus();

	GetDlgItem(IDC_BUTTON_WEBSITE)->SetFocus(); // This button is always enabled and clicking it by mistake is harmless.
	return FALSE; // We set the focus.
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CReader_x64_fixerDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CReader_x64_fixerDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CReader_x64_fixerDlg::OnBnClickedButtonWebsite()
{
	SHELLEXECUTEINFO sei = {0};
	sei.cbSize = sizeof(sei);
	sei.lpFile = L"http://www.pretentiousname.com/adobe_pdf_x64_fix/index.html";
	ShellExecuteEx(&sei);
}

void CReader_x64_fixerDlg::OnBnClickedCheckOffice2010beta()
{
	UpdateAppIDStatus();
}

void CReader_x64_fixerDlg::OnBnClickedOk()
{
	bool fOtherViewerIsAssigned = false;
	bool fLeaveAssocAlone = false;

	FixState stateA  = CheckAssoc(IDC_EDIT_ASSOC, &fOtherViewerIsAssigned);
	FixState state32 = CheckAppID(true,  0);
	FixState state64 = CheckAppID(false, 0);

	if (stateA == FS_NEEDFIX && fOtherViewerIsAssigned)
	{
		if (IDYES != AfxMessageBox(L"Another preview handler, not the Adobe one, is currently associated with PDF files. This tool can change this, and it will only affect preview handlers and will not modify what happens when you double-click PDF files. However, to undo this change afterwards you usually have to reinstall your other preview handler.\n\nDo you wish to assign the Adobe preview handler to PDF files?", MB_YESNO|MB_ICONQUESTION))
		{
			fLeaveAssocAlone = true;
			stateA = FS_HASFIX; // Skip that change.
		}
	}

	if (stateA != FS_ERROR && (stateA == FS_NEEDFIX || state32 == FS_NEEDFIX || state64 == FS_NEEDFIX))
	{
		if (stateA == FS_NEEDFIX)
		{
			ChangeAssoc();
		}

		if (state32 == FS_NEEDFIX)
		{
			ChangeAppID(true, true);
		}

		if (state64 == FS_NEEDFIX)
		{
			if (LeoHelpers::IsWindows7OrAbove() && !Office2010BetaMode())
			{
				DeleteKey64IfWin7();
			}
			else
			{
				CopyKey32to64IfNeeded();
				ChangeAppID(false, true);
			}
		}

		UpdateAppIDStatus();

		stateA  = CheckAssoc(IDC_EDIT_ASSOC, NULL);
		state32 = CheckAppID(true,  IDC_EDIT_32STATUS);
		state64 = CheckAppID(false, IDC_EDIT_64STATUS);

		if (stateA == FS_NEEDFIX && fLeaveAssocAlone)
		{
			stateA = FS_HASFIX; // User asked to ignore that so pretend that it's okay.
		}

		if (stateA == FS_NEEDFIX || state32 == FS_NEEDFIX || state64 == FS_NEEDFIX)
		{
			AfxMessageBox(L"Failed to apply the fix. There could be an issue with registry permissions or with the PDF registry data being in an unexpected configuration.", MB_OK|MB_ICONSTOP);
		}
	}
}

void CReader_x64_fixerDlg::OnBnClickedButtonUndo()
{
	FixState stateA  = CheckAssoc(IDC_EDIT_ASSOC, NULL);
	FixState state32 = CheckAppID(true,  0);
	FixState state64 = CheckAppID(false, 0);

	if (stateA != FS_ERROR && (state32 == FS_HASFIX || state64 == FS_HASFIX))
	{
		// Note: We don't "undo" the association fix as we don't know what the old association was.
		// Storing the old association creates a mess in the registry which few people will want.
		// If people are running this program at all they presumably want the Adobe PDF preview handler.
		// If people had a non-Adobe PDF preview handler and want to revert to it then they can reinstall it.

		if (state32 == FS_HASFIX)
		{
			ChangeAppID(true, false);
		}

		if (state64 == FS_HASFIX || state64 == FS_NEEDFIX)
		{
			ChangeAppID(false, false);
			DeleteKey64IfWin7();
		}

		UpdateAppIDStatus();

		state32 = CheckAppID(true,  IDC_EDIT_32STATUS);
		state64 = CheckAppID(false, IDC_EDIT_64STATUS);

		if ( state32 == FS_HASFIX
		||	(state64 == FS_HASFIX && (!LeoHelpers::IsWindows7OrAbove() || Office2010BetaMode())))
		{
			AfxMessageBox(L"Failed to undo the fix. There could be a registry permissions issue.", MB_OK|MB_ICONSTOP);
		}
	}
}

void CReader_x64_fixerDlg::UpdateAppIDStatus()
{
	FixState stateA  = CheckAssoc(IDC_EDIT_ASSOC, NULL);
	FixState state32 = CheckAppID(true,  IDC_EDIT_32STATUS);
	FixState state64 = CheckAppID(false, IDC_EDIT_64STATUS);

	bool fEnableFix  = (stateA != FS_ERROR && (stateA  == FS_NEEDFIX || state32 == FS_NEEDFIX || state64 == FS_NEEDFIX));
	bool fEnableUndo = (stateA != FS_ERROR && (state32 == FS_HASFIX || (state64 == FS_HASFIX && (!LeoHelpers::IsWindows7OrAbove() || Office2010BetaMode()))));

	GetDlgItem(IDOK)->EnableWindow(fEnableFix?TRUE:FALSE);
	GetDlgItem(IDC_BUTTON_UNDO)->EnableWindow(fEnableUndo?TRUE:FALSE);
}

void CReader_x64_fixerDlg::GetPdfRegKeys(std::vector< std::wstring > &vecKeyNames)
{
	vecKeyNames.clear();
	vecKeyNames.push_back(L".pdf");

	std::wstring strProgIdUnversioned;

	if (LeoHelpers::RegQueryStringValue(HKEY_CLASSES_ROOT, L".pdf", NULL, 0, &strProgIdUnversioned))
	{
		vecKeyNames.push_back(strProgIdUnversioned);

		std::wstring strCurVer = strProgIdUnversioned;
		strCurVer += L"\\CurVer";

		std::wstring strProgIdVersioned;

		if (LeoHelpers::RegQueryStringValue(HKEY_CLASSES_ROOT, strCurVer.c_str(), NULL, 0, &strProgIdVersioned))
		{
			vecKeyNames.push_back(strProgIdVersioned);
		}
	}

	vecKeyNames.push_back(L"SystemFileAssociations\\.pdf");

	// We do not look at the SystemFileAssociations\.pdf -> PerceivedType value. AFAIK no PDF preview handler
	// registers itself below there, and .pdf doesn't typically have a PerceivedType at all. Changing the
	// PerceivedType's preview handler without knowing more about it could be risky as it could be a preview
	// handler that handles more than just .PDF files and we could break viewing of other file types by assigning
	// Adobe Reader's handler to all of those types.
}

void CReader_x64_fixerDlg::GetPreviewHandlerName(const wchar_t *szClsid, std::wstring *pstrName)
{
	std::wstring strVal;

	// Looking in both places may be required on Vista (and is not needed but also harmless on Windows 7).
	if ((!LeoHelpers::RegQueryStringValue(HKEY_LOCAL_MACHINE, L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\PreviewHandlers", szClsid, KEY_WOW64_64KEY, &strVal) || strVal.empty())
	&&	(!LeoHelpers::RegQueryStringValue(HKEY_LOCAL_MACHINE, L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\PreviewHandlers", szClsid, KEY_WOW64_32KEY, &strVal) || strVal.empty()))
	{
		*pstrName = L"CLSID = \"";
		*pstrName += szClsid;
		*pstrName += L"\"";
	}
	else
	{
		*pstrName = L"\"";
		*pstrName += strVal;
		*pstrName += L"\"";
	}
}

bool CReader_x64_fixerDlg::Office2010BetaMode()
{
	return (::SendMessage(::GetDlgItem(m_hWnd,IDC_CHECK_OFFICE2010BETA), BM_GETCHECK, 0, 0) == BST_CHECKED);
}

CReader_x64_fixerDlg::FixState CReader_x64_fixerDlg::CheckAssoc(int nIdStatus, bool *pfOtherViewerIsAssigned)
{
	if (pfOtherViewerIsAssigned != NULL)
	{
		*pfOtherViewerIsAssigned = false;
	}

	if (!Is64BitWindows())
	{
		if (nIdStatus != 0)
		{
			GetDlgItem(nIdStatus)->SetWindowText(L"32-bit version of Windows detected. Fix is not needed.");
		}

		return FS_ERROR;
	}

	if (!LeoHelpers::RegDoesKeyExist(HKEY_LOCAL_MACHINE, s_szAdobePreviewHandlerClsidWithPath, KEY_WOW64_32KEY))
	{
		if (nIdStatus != 0)
		{
			GetDlgItem(nIdStatus)->SetWindowText(L"ERROR: Adobe Reader preview handler not found.");
		}

		return FS_ERROR;
	}

	if (!LeoHelpers::RegDoesKeyExist(HKEY_CLASSES_ROOT, L".pdf", 0))
	{
		if (nIdStatus != 0)
		{
			GetDlgItem(nIdStatus)->SetWindowText(L"ERROR: .PDF file type does not exist.");
		}

		return FS_ERROR;
	}

	std::vector< std::wstring > vecKeyNames;
	GetPdfRegKeys(vecKeyNames);

	bool fAnyAreSet = false;
	std::wstring strFirstWrongClsid;

	for (std::vector< std::wstring >::const_iterator keyNameIter = vecKeyNames.begin(); keyNameIter != vecKeyNames.end(); ++keyNameIter)
	{
		std::wstring strKey = *keyNameIter;

		// This GUID is the magic name for "preview handler".
		// The default value below this GUID will be the CLSID of the associated handler.
		strKey += L"\\ShellEx\\{8895b1c6-b41f-4c1c-a562-0d564250836f}";

		std::wstring strVal;

		if (LeoHelpers::RegQueryStringValue(HKEY_CLASSES_ROOT, strKey.c_str(), NULL, 0, &strVal))
		{
			fAnyAreSet = true;

			if (0 != _wcsicmp(strVal.c_str(), s_szAdobePreviewHandlerClsid)
			&&	strFirstWrongClsid.empty())
			{
				strFirstWrongClsid = strVal;
			}
		}
	}

	if (!fAnyAreSet)
	{
		if (nIdStatus != 0)
		{
			GetDlgItem(nIdStatus)->SetWindowText(L"NONE. No preview handler associated with PDF files.");
		}

		return FS_NEEDFIX;
	}

	if (!strFirstWrongClsid.empty())
	{
		if (nIdStatus != 0)
		{
			std::wstring strHandlerName;
			GetPreviewHandlerName(strFirstWrongClsid.c_str(), &strHandlerName);

			std::wstring strMsg = L"OTHER. ";
			strMsg += strHandlerName;

			GetDlgItem(nIdStatus)->SetWindowText(strMsg.c_str());
		}

		if (pfOtherViewerIsAssigned != NULL)
		{
			*pfOtherViewerIsAssigned = true;
		}

		return FS_NEEDFIX;
	}

	std::wstring strKey = L".pdf";
	strKey += L"\\ShellEx\\{8895b1c6-b41f-4c1c-a562-0d564250836f}";
	std::wstring strVal;

	if (!LeoHelpers::RegQueryStringValue(HKEY_CLASSES_ROOT, strKey.c_str(), NULL, 0, &strVal))
	{
		if (nIdStatus != 0)
		{
			GetDlgItem(nIdStatus)->SetWindowText(L"UNUSUAL. Association is correct but not in usual place.");
		}

		return FS_NEEDFIX;
	}

	if (nIdStatus != 0)
	{
		std::wstring strHandlerName;
		GetPreviewHandlerName(s_szAdobePreviewHandlerClsid, &strHandlerName);

		std::wstring strMsg = L"Correct. ";
		strMsg += strHandlerName;

		GetDlgItem(nIdStatus)->SetWindowText(strMsg.c_str());
	}

	return FS_HASFIX;
}

CReader_x64_fixerDlg::FixState CReader_x64_fixerDlg::CheckAppID(bool f32, int nIdStatus)
{
	std::wstring strVal;

	if (nIdStatus != 0)
	{
		assert((f32 && nIdStatus == IDC_EDIT_32STATUS) || (!f32 && nIdStatus == IDC_EDIT_64STATUS));
	}

	if (!Is64BitWindows())
	{
		if (nIdStatus != 0)
		{
			GetDlgItem(nIdStatus)->SetWindowText(L"32-bit version of Windows detected. Fix is not needed.");
		}

		return FS_ERROR;
	}

	const REGSAM sam = f32 ? KEY_WOW64_32KEY : KEY_WOW64_64KEY;

	if (!LeoHelpers::RegQueryStringValue(HKEY_LOCAL_MACHINE, s_szAdobePreviewHandlerClsidWithPath, L"AppId", sam, &strVal))
	{
		if (f32 || CheckAppID(true, 0) == FS_ERROR)
		{
			if (nIdStatus != 0)
			{
				GetDlgItem(nIdStatus)->SetWindowText(L"VALUE NOT FOUND. (Is Adobe Reader installed?)");
			}
			return FS_ERROR;
		}
		else if (!LeoHelpers::IsWindows7OrAbove() || Office2010BetaMode())
		{
			// While normal for Windows 7 (unless Adobe Reader was installed on Vista and the machine then upgraded to Windows 7),
			// 64-bit Office 2010 *BETA* will have problems using the preview handler unless it's registered in the 64-bit registry.
			if (nIdStatus != 0)
			{
				GetDlgItem(nIdStatus)->SetWindowText(L"VALUE NOT FOUND. Click \"Apply Fix\" to rectify.");
			}
			return FS_NEEDFIX;
		}
		else
		{
			if (nIdStatus != 0)
			{
				GetDlgItem(nIdStatus)->SetWindowText(L"Value not found. (This is normal on Windows 7.)");
			}

			return FS_HASFIX;
		}
	}
	
	bool fIsKnownWrongAppId = (0 == _wcsicmp(strVal.c_str(), s_szAppIDWrong));
	bool fIsKnownRightAppId = (0 == _wcsicmp(strVal.c_str(), s_szAppIDRight));

	if (!fIsKnownWrongAppId && !fIsKnownRightAppId)
	{
		if (nIdStatus != 0)
		{
			GetDlgItem(nIdStatus)->SetWindowText(L"UNEXPECTED VALUE. Fix may be unsafe and won't be applied.");
		}

		return FS_ERROR;
	}

	if (!f32 && LeoHelpers::IsWindows7OrAbove() && !Office2010BetaMode())
	{
		if (nIdStatus != 0)
		{
			GetDlgItem(nIdStatus)->SetWindowText(L"VALUE UNWANTED. (Should delete on Win7 unless Office2010 beta.)");
		}

		return FS_NEEDFIX;
	}

	if (fIsKnownWrongAppId)
	{
		if (nIdStatus != 0)
		{
			GetDlgItem(nIdStatus)->SetWindowText(L"VALUE IS INCORRECT. Click \"Apply Fix\" to rectify.");
		}

		return FS_NEEDFIX;
	}

	if (fIsKnownRightAppId)
	{
		if (nIdStatus != 0)
		{
			GetDlgItem(nIdStatus)->SetWindowText(L"Value is correct.");
		}

		return FS_HASFIX;
	}

	if (nIdStatus != 0)
	{
		GetDlgItem(nIdStatus)->SetWindowText(L"Error detecting state. Please report to leo@ox.compsoc.net");
	}

	return FS_ERROR;
}

void CReader_x64_fixerDlg::ChangeAssoc()
{
	std::vector< std::wstring > vecKeyNames;
	GetPdfRegKeys(vecKeyNames);

	bool fAnyAreSet = false;

	for (std::vector< std::wstring >::const_iterator keyNameIter = vecKeyNames.begin(); keyNameIter != vecKeyNames.end(); ++keyNameIter)
	{
		std::wstring strKey = *keyNameIter;

		// This GUID is the magic name for "preview handler".
		// The default value below this GUID will be the CLSID of the associated handler.
		strKey += L"\\ShellEx\\{8895b1c6-b41f-4c1c-a562-0d564250836f}";

		std::wstring strVal;

		if (LeoHelpers::RegQueryStringValue(HKEY_CLASSES_ROOT, strKey.c_str(), NULL, 0, &strVal))
		{
			fAnyAreSet = true;

			if (0 != _wcsicmp(strVal.c_str(), s_szAdobePreviewHandlerClsid))
			{
				LeoHelpers::RegSetStringValue(HKEY_CLASSES_ROOT, strKey.c_str(), NULL, 0, s_szAdobePreviewHandlerClsid);
			}
		}
	}

//	if (!fAnyAreSet) <-- Actually, always do this as it's what Adobe themselves do and a good, redundant fallback.
	{
		// There isn't a PDF preview handler association so create one in the place that it normally is.
		std::wstring strKey = L".pdf";
		strKey += L"\\ShellEx\\{8895b1c6-b41f-4c1c-a562-0d564250836f}";
		LeoHelpers::RegSetStringValue(HKEY_CLASSES_ROOT, strKey.c_str(), NULL, 0, s_szAdobePreviewHandlerClsid);
	}

	// One thing we don't really cope with here is where the handler is wrong via HKEY_CURRENT_USER.
	// Dealing with that, and knowing exactly how to correct it without nuking the PDF file type, doesn't seem obvious.
	// (For a start, we're running as admin but the user account could be someone else and invisible to us.
	// If it's a problem then there may need to be a non-admin mode for the program, but hopefully this issue is
	// purely theoretical and the PDF file type will alawys exist via HKEY_LOCAL_MACHINE.)
}

void CReader_x64_fixerDlg::ChangeAppID(bool f32, bool fFix)
{
	const REGSAM   sam     = f32  ? KEY_WOW64_32KEY : KEY_WOW64_64KEY;
	const wchar_t *szAppID = fFix ? s_szAppIDRight  : s_szAppIDWrong;

	LeoHelpers::RegSetStringValue(HKEY_LOCAL_MACHINE, s_szAdobePreviewHandlerClsidWithPath, L"AppId", sam, szAppID);
}

void CReader_x64_fixerDlg::CopyKey32to64IfNeeded()
{
	std::wstring strVal;

	if ( LeoHelpers::RegQueryStringValue(HKEY_LOCAL_MACHINE, s_szAdobePreviewHandlerClsidWithPath, L"AppId", KEY_WOW64_32KEY, &strVal)
	&&	!LeoHelpers::RegQueryStringValue(HKEY_LOCAL_MACHINE, s_szAdobePreviewHandlerClsidWithPath, L"AppId", KEY_WOW64_64KEY, &strVal))
	{
		LeoHelpers::RegCopyKey(HKEY_LOCAL_MACHINE, KEY_WOW64_32KEY, HKEY_LOCAL_MACHINE, KEY_WOW64_64KEY, s_szAdobePreviewHandlerClsidWithPath);
	}
}

void CReader_x64_fixerDlg::DeleteKey64IfWin7()
{
	// Under Windows 7, unless Adobe Reader was installed on Vista and that then upgraded to 7, it's normal for only the 32-bit
	// CLSID to exist. As part of undoing our changes we will delete the 64-bit CLSID if on Win 7.
	if (LeoHelpers::IsWindows7OrAbove())
	{
		std::wstring strVal;

		// Sanity check: Make sure the **32-bit** key exists before deleting the 64-bit key, in case in the future only the 64-bit one exists.
		// We don't want to delete the only copy of the key.
		if (LeoHelpers::RegQueryStringValue(HKEY_LOCAL_MACHINE, s_szAdobePreviewHandlerClsidWithPath, L"AppId", KEY_WOW64_32KEY, &strVal))
		{
			HKEY hkDest = NULL;
			
			// Delete the **64-bit** key.
			LeoHelpers::RegDeleteKey(HKEY_LOCAL_MACHINE, NULL, s_szAdobePreviewHandlerClsidWithPath, KEY_WOW64_64KEY, true);
		}
	}
}
